/**
 ******************************************************************************
 * @file    main.c
 * @author  psavr@gmx.ch
 * @version V1.0
 * @date    20-Novermer-2012
 * @brief   Demo-Application for STM32F4-Discovery Board
 *          including printf() for USART2 USART3 or USART6
 *          including generic I2C functions
 ******************************************************************************
 * @attention
 *
 * This Example is maintained under the PsAvr "BEERWARE-LICENSE"
 * Feel free to use it, to modify it and to distribute it. If you like
 * it and we meet us once in a pub or a bar, feel free to spend me a beer!
 *
 * I would appreciate it very much, if you would return any corrections
 * and/or improvements back to me and/or to the CooCox community.
 ******************************************************************************
 */

/*---------------------------- Include ---------------------------------------*/

#include "main.h"
#include "usart.h"
#include "i2c.h"

/*---------------------------- Symbol Define -------------------------------*/

#define GPIO_TIMEOUT      (0x100000)  // timeout for blinking LEDs
#define I2C_EEPROM_ADDR   (0xA0)     // EEPROM I2C-Bus device address
#define I2C_EEPROM_PAGE   (32)       // EEPROM memory page size
/*---------------------------- Variable Define -----------------------------*/

/*---------------------------- Function Prototypes -------------------------*/

void store_to_eeprom(char* myString, uint16_t addr_ptr);
void read_from_eeprom(char* myString, uint16_t addr_ptr);
void LED_Init(void);
void Shell(void);

/*---------------------------- Function Code -------------------------------*/

void LED_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  /* GPIOD Periph clock enable */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
  /* Configure PD12, PD13, PD14 and PD15 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = (GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);
}

/**
 *******************************************************************************
 * @brief       "Terminal" task code
 * @param[in]   None
 * @param[out]  None
 * @retval      None
 * @par Description
 * @details    This task is the terminal-demon for the stdio
 *******************************************************************************
 */

void store_to_eeprom(char* myString, uint16_t addr_ptr)
{
  printf("Write String: >%s< to EEPROM-Addr[0x%04X]\n", myString, addr_ptr);
  while (I2C_CheckDevice(I2C_EEPROM_ADDR) != OK);  // wait until EEPROM is ready
  I2C_WriteTransfer(I2C_EEPROM_ADDR, myString, strlen(myString)+1, addr_ptr, sizeof(addr_ptr));
}

void read_from_eeprom(char* myString, uint16_t addr_ptr)
{
  while (I2C_CheckDevice(I2C_EEPROM_ADDR) != OK);  // wait until EEPROM is ready
  I2C_ReadTransfer(I2C_EEPROM_ADDR, myString, I2C_EEPROM_PAGE, addr_ptr, sizeof(addr_ptr));
  printf("Read String: >%s< from  EEPROM-Addr[0x%04X]\n", myString, addr_ptr);
}


void Shell(void)
{
  static char newCommand[128];
  static char myString[I2C_EEPROM_PAGE] = {"Default String"};
  uint32_t led_pin = GPIO_Pin_12;
  uint32_t timer = GPIO_TIMEOUT;
  //---------------------------------------------------------------------------------------------------
  if (I2C_CheckDevice(I2C_EEPROM_ADDR) != OK)
  {
    printf("2C-Bus device with address[0x%02X] is not responding\n",I2C_EEPROM_ADDR);
  }
  else
  {
    store_to_eeprom("String-A @ EEPROM-Addr[0x0020]",0x0020);
    store_to_eeprom("String-B @ EEPROM-Addr[0x0040]",0x0040);
    store_to_eeprom("String-C @ EEPROM-Addr[0x0060]",0x0060);
    store_to_eeprom("String-D @ EEPROM-Addr[0x0080]",0x0080);
  }
  //---------------------------------------------------------------------------------------------------
  printf("Terminal-Task: started\n>");
  fflush(0);
  while (1)
  {
    //---------------------------------------------------------------------
    if (UART_GetLine(newCommand))
    {
      //---------------------------------------------------------------------
      if      (!strcmp(newCommand, "w1")) store_to_eeprom(myString, 0x0020);
      else if (!strcmp(newCommand, "w2")) store_to_eeprom(myString, 0x0040);
      else if (!strcmp(newCommand, "w3")) store_to_eeprom(myString, 0x0060);
      else if (!strcmp(newCommand, "w4")) store_to_eeprom(myString, 0x0080);
      //-----------------------------------------------------------------
      else if (!strcmp(newCommand, "r1")) read_from_eeprom(myString, 0x0020);
      else if (!strcmp(newCommand, "r2")) read_from_eeprom(myString, 0x0040);
      else if (!strcmp(newCommand, "r3")) read_from_eeprom(myString, 0x0060);
      else if (!strcmp(newCommand, "r4")) read_from_eeprom(myString, 0x0080);
      //---------------------------------------------------------------------
      else if (*newCommand != '\r')
      {
        strncpy(myString, newCommand, sizeof(myString));
        myString[sizeof(myString) - 1] = '\0';
        printf(">New String: >%s<\n", myString);
      }
      //---------------------------------------------------------------------
      printf(">");
      fflush(0);
    }
    //---------------------------------------------------------------------
    if (!timer--)
    {
      timer = GPIO_TIMEOUT;
      GPIO_ResetBits(GPIOD, led_pin);
      led_pin = (led_pin << 1);
      if (led_pin > GPIO_Pin_15)
      {
        led_pin = GPIO_Pin_12;
      }
      GPIO_SetBits(GPIOD, led_pin);
    }
    //---------------------------------------------------------------------
  }
}

/*---------------------------- Main Code -------------------------------*/

int main(void)
{
  LED_Init();
  UART_Init();
  I2C_BusInit();
  printf("\n-----------------------------------\n");
  printf("STM32F4-Discovery Board is booting!\n");
  printf("UART and I2C-Bus Demo for STM32F4  \n");
  printf("-----------------------------------\n");
  while (1)
  {
    Shell();
    NVIC_SystemReset(); // nope
  }
}

